import os, sys
import platform
from ctypes import resize, cdll, c_void_p, c_float, c_int, c_uint, c_byte
from ctypes import Structure, byref, pointer, CFUNCTYPE, c_char_p, c_char
from ctypes import c_long, POINTER, c_bool, c_uint64

# Node Types
NT_UNKNOWN							= 0
NT_MESH								= 1
NT_GEOMETRY_MATERIAL_MAP			= 2
NT_GEOMETRY_GROUP					= 3
NT_GEOMETRY_PLACEMENT				= 4
NT_GEOMETRY_SCATTER					= 5
NT_FLOAT							= 6
DEPRECATED_NT_FLOAT2				= 7
DEPRECATED_NT_FLOAT3				= 8
NT_INT								= 9
DEPRECATED_NT_INT2					= 10
NT_BOOL								= 11
NT_INT2RESOLUTION					= 12
NT_THINLENSCAMERA					= 13
NT_DAYLIGHTENVIRONMENT				= 14
NT_CAMERAIMAGER						= 15
NT_MATERIAL_GLOSSY					= 16
NT_MATERIAL_DIFFUSE					= 17
NT_MATERIAL_SPECULAR				= 18
NT_MATERIAL_MIX						= 19
NT_MATERIAL_PORTAL					= 20
DEPRECATED_NT_CAMERARESPONSEDIST    = 21
NT_TURBULENCETEXTURE				= 22
NT_PMCKERNEL						= 23
NT_DIRECTLIGHTINGKERNEL				= 24
NT_PATHTRACINGKERNEL				= 25
NT_INFOCHANNELKERNEL				= 26
NT_TRANSFORM						= 27
NT_TRANSFORM_SCALE					= 28
NT_TRANSFORM_ROTATION				= 29
NT_FLOAT3DAYLIGHTSYSTEM				= 30
NT_FLOATTEXTURE						= 31
NT_TEXTURE_GAUSSIANSPECTRUM			= 32
NT_TEXTURE_RGBSPECTRUM				= 33
NT_TEXTURE_IMAGE					= 34
NT_TEXTURE_ALPHAIMAGE				= 35
NT_TEXTURE_FLOATIMAGE				= 36
NT_TEXTUREENVIRONMENT				= 37
NT_TEXTURE_MIX						= 38
NT_TEXTURE_MULTIPLY					= 39
NT_TEXTURE_COSINEMIX				= 40
NT_TEXTURE_CLAMP					= 41
NT_TEXTURE_SINEWAVE					= 42
NT_TEXTURE_TRIANGLEWAVE				= 43
NT_TEXTURE_SAWWAVE					= 44
NT_TEXTURE_CHECKS					= 45
NT_TEXTURE_INVERT					= 46
NT_TEXTURE_MARBLE					= 47
NT_TEXTURE_RGFRACTAL				= 48
NT_TEXTURE_GRADIENT					= 49
NT_TEXTURE_FALLOFF					= 50
NT_TEXTURE_COLOR_CORRECT			= 51
DEPRECATED_NT_EMISSION_NULL         = 52
NT_EMISSION_BLACKBODY				= 53
NT_EMISSION_TEXTURE					= 54
NT_APPLICATIONPREFERENCES			= 55
NT_RENDER_TARGET					= 56
NT_LISTINT							= 57
NT_MEDIUM_ABSORPTION				= 58 
NT_MEDIUM_SCATTERING				= 59
NT_PHASEFUNCTION_SCHLICK			= 60
NT_POSTPROCESSING					= 61
NT_PANORAMIC_CAMERA					= 62
NT_TEXTURE_DIRT						= 63
NT_OBJECTLAYER_MAP                  = 64
NT_OBJECTLAYER                      = 65
NT_TRANSFORM_2D                     = 66
NT_TRANSFORM_VALUE                  = 67
NT_ANNOTATION                       = 68
NT_KERN_MATPREVIEW                  = 69
NT_PROJECT_SETTINGS                 = 70
NT_REMOTE_APP_PREFS                 = 71
NT_SPLIT_PANE                       = 72
NT_WORK_PANE                        = 73
NT_PROJ_CYLINDRICAL                 = 74
NT_PROJ_LINEAR                      = 75
NT_PROJ_PERSPECTIVE                 = 76
NT_PROJ_SPHERICAL                   = 77
NT_PROJ_UVW                         = 78
NT_PROJ_BOX                         = 79
NT_DISPLACEMENT                     = 80
NT_TEX_RANDOMCOLOR                  = 81
NT_STRING                           = 84
NT_SCRIPT_INPUT                     = 85
NT_RENDER_PASSES                    = 86
NT_TEX_NOISE                        = 87
NT_FILE                             = 88
NT_TEX_SIDE                         = 89
NT_RENDER_LAYER                     = 90
NT_GEO_VOLUME                       = 91
NT_ORC_VERSION                      = 92
NT_IMPORT_VDB_PREFS                 = 93
NT_CAM_BAKING                       = 94 # 3.0.1
NT_VOLUME_RAMP                      = 95 # Octane 3.0.5
NT_UNUSED                           = 96 # Octane 3.0.5
NT_CAM_CUSTOM                       = 97 # Octane 3.0.5
NT_MED_VOLUME                       = 98 # Octane 3.0.5
NT_ANIMATION_SETTINGS               = 99 # 3.0.10
NT_FILM_SETTINGS                    = 100 # 3.0.15

ApiNodeTypesStr = {                       # internal name           pin type            label
    NT_UNKNOWN							: ["unknown",               "Unknown",          "Unknown"],                 # 0
    NT_MESH								: ["mesh",                  "Geometry" ,        "Mesh"],                    # 1
    NT_GEOMETRY_MATERIAL_MAP			: ["material map",          "Geometry",         "Material Map"],            # 2
    NT_GEOMETRY_GROUP					: ["geometrygroup",         "Geometry",         "Geometry Group"],          # 3
    NT_GEOMETRY_PLACEMENT				: ["placement",             "Geometry",         "Placement"],               # 4
    NT_GEOMETRY_SCATTER					: ["scatter",               "Geometry",         "Scatter"],                 # 5
    NT_FLOAT							: ["float",                 "Float",            "Float Value"],             # 6
    DEPRECATED_NT_FLOAT2				: ["float2",                "_Float",           ""],                        # 7
    DEPRECATED_NT_FLOAT3				: ["float3",                "_Float",           ""],                        # 8
    NT_INT								: ["int",                   "Int",              "Int Value"],               # 9
    DEPRECATED_NT_INT2					: ["int2",                  "_Int",             "Int2"],                    # 10
    NT_BOOL								: ["bool",                  "Bool",             "Bool Value"],              # 11
    NT_INT2RESOLUTION					: ["int2resolution",        "Int",              "Image Resolution"],        # 12
    NT_THINLENSCAMERA					: ["thinlens",              "Camera",           "Thin Lens Camera"],        # 13
    NT_DAYLIGHTENVIRONMENT				: ["daylight",              "Environment",      "Daylight"],                # 14
    NT_CAMERAIMAGER						: ["camera",                "Imager",           "Camera"],                  # 15
    NT_MATERIAL_GLOSSY					: ["glossy",                "Material",         "Glossy"],                  # 16
    NT_MATERIAL_DIFFUSE					: ["diffuse",               "Material",         "Diffuse"],                 # 17
    NT_MATERIAL_SPECULAR				: ["specular",              "Material",         "Specular"],                # 18
    NT_MATERIAL_MIX						: ["material mix",          "Material",         "Material Mix"],            # 19
    NT_MATERIAL_PORTAL					: ["portal",                "Material",         "Portal"],                  # 20 
    DEPRECATED_NT_CAMERARESPONSEDIST	: ["camera response",       "Response",         ""],                        # 21
    NT_TURBULENCETEXTURE				: ["turbulence",            "Texture",          "Turbulence"],              # 22
    NT_PMCKERNEL						: ["pmc",                   "Kernel",           "PMC"],                     # 23
    NT_DIRECTLIGHTINGKERNEL				: ["directlighting",        "Kernel",           "Direct Lighting"],         # 24
    NT_PATHTRACINGKERNEL				: ["pathtracing",           "Kernel",           "Path Tracing"],            # 25
    NT_INFOCHANNELKERNEL				: ["deep_channel_kernel",   "Kernel",           "Infochannels Kernel"],     # 26 
    NT_TRANSFORM						: ["transform",             "Transform",        "3D Transformation"],       # 27
    NT_TRANSFORM_SCALE					: ["scale",                 "Transform",        "3D Scale"],                # 28 
    NT_TRANSFORM_ROTATION				: ["rotation",              "Transform",        "3D Rotation"],             # 29
    NT_FLOAT3DAYLIGHTSYSTEM				: ["float3daylightsystem",  "Float",            "Sun Direction"],           # 30
    NT_FLOATTEXTURE						: ["floattexture",          "Texture",          "Greyscale Color"],         # 31
    NT_TEXTURE_GAUSSIANSPECTRUM			: ["gaussianspectrum",      "Texture",          "Gaussian Spectrum"],       # 32
    NT_TEXTURE_RGBSPECTRUM				: ["RGBspectrum",           "Texture",          "RGB Color"],               # 33
    NT_TEXTURE_IMAGE					: ["image",                 "Texture",          "RGB Image"],               # 34
    NT_TEXTURE_ALPHAIMAGE				: ["alphaimage",            "Texture",          "Alpha Image"],             # 35
    NT_TEXTURE_FLOATIMAGE				: ["floatimage",            "Texture",          "Greyscale Image"],         # 36
    NT_TEXTUREENVIRONMENT				: ["texture environment",   "Environment",      "Texture Environment"],     # 37
    NT_TEXTURE_MIX						: ["mix",                   "Texture",          "Mix"],                     # 38
    NT_TEXTURE_MULTIPLY					: ["multiply",              "Texture",          "Multiply"],                # 39
    NT_TEXTURE_COSINEMIX				: ["cosinemix",             "Texture",          "Cosine Mix"],              # 40
    NT_TEXTURE_CLAMP					: ["clamptexture",          "Texture",          "Clamp"],                   # 41
    NT_TEXTURE_SINEWAVE					: ["sinewave",              "Texture",          "Sine Wave"],               # 42
    NT_TEXTURE_TRIANGLEWAVE				: ["trianglewave",          "Texture",          "Triangle Wave"],           # 43
    NT_TEXTURE_SAWWAVE					: ["sawwave",               "Texture",          "Saw Wave"],                # 44
    NT_TEXTURE_CHECKS					: ["checks",                "Texture",          "Checks"],                  # 45
    NT_TEXTURE_INVERT					: ["invert",                "Texture",          "Invert"],                  # 46
    NT_TEXTURE_MARBLE					: ["marble",                "Texture",          "Marble"],                  # 47
    NT_TEXTURE_RGFRACTAL				: ["ridgedfractal",         "Texture",          "Ridged Fractal"],          # 48
    NT_TEXTURE_GRADIENT					: ["gradient",              "Texture",          "Gradient"],                # 49
    NT_TEXTURE_FALLOFF					: ["falloff map",           "Texture",          "Falloff Map"],             # 50
    NT_TEXTURE_COLOR_CORRECT			: ["color correction",      "Texture",          "Color Correction"],        # 51
    DEPRECATED_NT_EMISSION_NULL			: ["null emission",         "_Emission",        "Null Emission"],           # 52 (deprecated)
    NT_EMISSION_BLACKBODY				: ["blackbody",             "Emission",         "Black Body"],              # 53
    NT_EMISSION_TEXTURE					: ["texture emission",      "Emission",         "Texture Emission"],        # 54
    NT_APPLICATIONPREFERENCES			: ["apppreferences",        "Unknown",          "Application Preferences"], # 55
    NT_RENDER_TARGET					: ["rendertarget",          "Unknown",          "Rendertarget"],            # 56
    NT_LISTINT							: ["list int",              "Int",              "Enum"],                    # 57
    NT_MEDIUM_ABSORPTION				: ["absorption",            "Medium",           "Absorption"],              # 58
    NT_MEDIUM_SCATTERING				: ["scattering",            "Medium",           "Scattering"],              # 59
    NT_PHASEFUNCTION_SCHLICK			: ["schlick",               "PhaseFunction",    "Schlick"],                 # 60
    NT_POSTPROCESSING					: ["postprocessing",        "PostProcess",      "Post Processing"],         # 61
    NT_PANORAMIC_CAMERA					: ["panoramic",             "Camera",           "Panoramic Camera"],        # 62
    NT_TEXTURE_DIRT						: ["dirt",                  "Texture",          "Dirt"],                    # 63
    NT_OBJECTLAYER_MAP                  : ["objectlayermap",        "Unknown",          "Object Layer Map"],        # 64
    NT_OBJECTLAYER                      : ["objectlayer",           "Unknown",          "Object Layer"],            # 65
    NT_TRANSFORM_2D                     : ["2dtransformation",      "Transform",        "2D Transformation"],       # 66
    NT_TRANSFORM_VALUE                  : ["transvalue",            "_Transform",       "Transformation Value"],    # 67
    NT_ANNOTATION                       : ["annotation",            "Unknown",          ""],                        # 68
    NT_KERN_MATPREVIEW                  : ["matpreview",            "Unknown",          ""],                        # 69
    NT_PROJECT_SETTINGS                 : ["projectsettings",       "Unknown",          ""],                        # 70
    NT_REMOTE_APP_PREFS                 : ["remoteappprefs",        "Unknown",          ""],                        # 71
    NT_SPLIT_PANE                       : ["splitpane",             "Unknown",          ""],                        # 72
    NT_WORK_PANE                        : ["workpane",              "Unknown",          ""],                        # 73
    NT_PROJ_CYLINDRICAL                 : ["cylindrical",           "Projection",       "Cylindrical"],             # 74
    NT_PROJ_LINEAR                      : ["linear",                "Projection",       "Linear"],                  # 75
    NT_PROJ_PERSPECTIVE                 : ["perspective",           "Projection",       "Perspective"],             # 76
    NT_PROJ_SPHERICAL                   : ["spherical",             "Projection",       "Spherical"],               # 77
    NT_PROJ_UVW                         : ["uv",                    "Projection",       "UV"],                      # 78
    NT_PROJ_BOX                         : ["box",                   "Projection",       "Box"],                     # 79
    NT_DISPLACEMENT                     : ["displacement",          "Displacement",     "Displacement"],            # 80
    NT_TEX_RANDOMCOLOR                  : ["random color",          "Texture",          "Random Color"],            # 81
    82                                  : ["alembic prefs",         "Unknown",          ""],                        # 82
    83                                  : ["obj prefs",             "Unknown",          ""],                        # 83
    NT_STRING                           : ["string",                "String",           "String"],                  # 84
    NT_SCRIPT_INPUT                     : ["script",                "Script",           "Script"],                  # 85
    NT_RENDER_PASSES                    : ["render passes",         "RenderPasses",     "Render Passes"],           # 86
    NT_TEX_NOISE                        : ["noise"  ,               "Texture",          "Noise"],                   # 87
    NT_FILE                             : ["file",                  "Unknown",          "File"],                    # 88
    NT_TEX_SIDE                         : ["side",                  "Texture",          "Polygon Side"],            # 89 
    NT_RENDER_LAYER                     : ["render layer",          "RenderLayer",      "Render Layer"]             # 90
}

# Pin Types
PT_UNKNOWN				= 0
PT_BOOL					= 1
PT_FLOAT				= 2
PT_INT					= 3
PT_TRANSFORM			= 4
PT_TEXTURE				= 5
PT_EMISSION				= 6
PT_MATERIAL				= 7
PT_CAMERA				= 8
PT_ENVIRONMENT			= 9
PT_IMAGER				= 10
PT_KERNEL				= 11
PT_GEOMETRY				= 12
PT_MEDIUM				= 13
PT_PHASEFUNCTION		= 14
PT_FILM_SETTINGS        = 15
PT_ENUM					= 16
PT_OBJECTLAYER			= 17
PT_POSTPROCESSING		= 18
PT_RENDERTARGET			= 19
PT_WORK_PANE			= 20
PT_PROJECTION			= 21
PT_DISPLACEMENT			= 22
PT_STRING				= 23
PT_RENDERPASSES			= 24
PT_RENDERLAYER			= 25
PT_VOLUME_RAMP			= 26
PT_ANIMATION_SETTINGS   = 27

STRING                  = c_char_p
OCTANE_NODE_PTR         = c_void_p
c_float3                = c_float * 3

# ctypes structs
class ResultStatistics(Structure):
    _fields_ = [('sizeX',               c_int),
                ('sizeY',               c_int),
                ('samplesPerPixel',     c_int),
                ('maxSamplesPerPixel',  c_int),
                ('samplesPerSecond',    c_float),
                ('renderTime',          c_float),
                ('changeLevel',         c_int),
                ('finished',            c_bool)]
                
class PinInfo(Structure):
    _fields_ = [('mType',               c_int),
                ('mComponentCount',     c_int),
                ('mDefaultValues',      c_float3),
                ('mMinValue',           c_float),
                ('mMaxValue',           c_float),
                ('mIsLogLinCapable',    c_bool),
                ('mUseLogByDefault',    c_bool),
                ('mFullPinName',        c_char * 50), 
                ('mToolTip',            STRING)]                         
                
class Float3(Structure):
    _fields_ = [('x',                   c_float),
                ('y',                   c_float),
                ('z',                   c_float)]

class Int3(Structure):
    _fields_ = [('x',                   c_int),
                ('y',                   c_int),
                ('z',                   c_int)]

# Node Utils
class NodeUtils(object):
    def getNodeInternalType(self, node):
        if bool(node) == 0:
            print "Null node passed to getNodeInternalType"
        else:
            node_type = GetNodeType(node)
            if node_type > 0 and node_type <= len(ApiNodeTypesStr.keys()):
                return ApiNodeTypesStr[node_type][0]
        return "unknown"

    def getNodeType(self, node):
        if bool(node) == 0:
            print "Null node passed to getNodeType"
        else:
            node_type = GetNodeType(node)
            if node_type > 0 and node_type <= len(ApiNodeTypesStr.keys()):
                return ApiNodeTypesStr[node_type][1]
        return "unknown"

    def getNodeLabel(self, node):
        if bool(node) == 0:
            print "Null node passed to getNodeLabel"
        else:
            node_type = GetNodeType(node)
            if node_type > 0 and node_type <= len(ApiNodeTypesStr.keys()):
                return ApiNodeTypesStr[node_type][2]
        return "unknown"

    def getAllNodesByType(self, rndr_node, 
                    node_type, all_nodes={}):
        pins = GetPinCount(rndr_node)
        for pin in xrange(pins):
            node = GetPinNode(rndr_node, pin)
            if bool(node) != 0:
                if self.getNodeType(node) == node_type:
                    node_label = self.getNodeLabel(node)
                    node_int_name = self.getNodeInternalType(node)
                    all_nodes[node] = (node_int_name, node_label)
                self.getAllNodesByType(node, node_type, all_nodes)
        return all_nodes

    def getAllNodesByInternalName(self, rndr_node, 
                                    node_int_name):
        filtered_nodes = dict()
        node_type = [ApiNodeTypesStr[t][1] for t in ApiNodeTypesStr \
                            if ApiNodeTypesStr[t][0] == node_int_name]
        if not node_type:
            return
        nodes = self.getAllNodesByType(rndr_node, node_type[0])
        for n in nodes:
            if nodes[n][0] == node_int_name:
                filtered_nodes[n] = nodes[n]
        return filtered_nodes



# Load Octane Library
class LoadOctaneLibrary(object):
    def __init__(self):
        self.lib_name = "OctanePythonWrapper.dll"
        self.path_name = self.getPath()

    def getPath(self):
        path, filename = os.path.split(__file__)
        return path
        
    def loadOctaneLibrary(self):
        if sys.platform != "cli" and sys.platform != "win32":
            print "Fatal Error importing octane.py. Unknown Platform : " + str(sys.platform)
            return

        if platform.architecture()[0] != "64bit":
            print "Fatal Error importing octane.py. 64bit only supported"
            return

        try:
            bundle_path = os.path.join(self.path_name, self.lib_name)
            retval = cdll.LoadLibrary(bundle_path)
            return retval
        except:
            print "Octane library failed to load"

# Load library
lol = LoadOctaneLibrary()
m_OctaneLib = lol.loadOctaneLibrary()

# Methods
if m_OctaneLib != None:
    # Rendering controls
    RestartRendering                        = m_OctaneLib.RestartRendering
    PauseRendering                          = m_OctaneLib.PauseRendering        
    ContinueRendering                       = m_OctaneLib.ContinueRendering
    IsRenderingPaused                       = m_OctaneLib.IsRenderingPaused
    IsRenderingPaused.restype               = c_bool                    # True if paused
    
    # Project
    ResetProject                            = m_OctaneLib.ResetProject
    ResetProject.restype                    = c_bool                    # Succeeded

    SaveImage                               = m_OctaneLib.SaveImage
    SaveImage.argtypes                      = [STRING, c_int, c_int]    # Filename, ImageSaveType, RenderPass
    SaveImage.restype                       = c_bool                    # Succeeded
        
    SaveImageRenderPasses                   = m_OctaneLib.SaveImageRenderPasses
    SaveImage.argtypes                      = [STRING]                  # Filename
        
    SaveProjectAs                           = m_OctaneLib.SaveProjectAs
    SaveProjectAs.argtypes                  = [STRING]                  # Filename
    SaveProjectAs.restype                   = c_bool                    # Succeeded
    
    SavePackage                             = m_OctaneLib.SavePackage
    SavePackage.argtypes                    = [STRING]                  # Filename
    SavePackage.restype                     = c_bool                    # Succeeded
        
    Exit                                    = m_OctaneLib.Exit          # Not required
    
    IsStarted                               = m_OctaneLib.IsStarted
    IsStarted.restype                       = c_bool                    # True if Octane started
    
    UpdateChangeManager                     = m_OctaneLib.UpdateChangeManager
    
    # Cuda Devices    
    ResetAllCudaDevices                     = m_OctaneLib.ResetAllCudaDevices
    
    GetNumDevices                           = m_OctaneLib.GetNumDevices
    GetNumDevices.restype                   = c_int                      # The number of cuda devices
    
    GetDeviceComputeModel                   = m_OctaneLib.GetDeviceComputeModel
    GetDeviceComputeModel.argtypes          = [c_int]
    GetDeviceComputeModel.restype           = c_int
    
    GetDeviceName                           = m_OctaneLib.GetDeviceName
    GetDeviceName.argtypes                  = [c_int]
    GetDeviceName.restype                   = STRING
        
    GetDeviceRenderActivity                 = m_OctaneLib.GetDeviceRenderActivity
    GetDeviceRenderActivity.argtypes        = [c_int]
    GetDeviceRenderActivity.restype         = c_bool
    
    SetDeviceRenderActivity                 = m_OctaneLib.SetDeviceRenderActivity
    SetDeviceRenderActivity.argtypes        = [c_int, c_int]

    GetDeviceUsesRenderPriority             = m_OctaneLib.GetDeviceUsesRenderPriority
    GetDeviceUsesRenderPriority.argtypes    = [c_int]
    GetDeviceUsesRenderPriority.restype     = c_bool
    
    SetDeviceUsesRenderPriority             = m_OctaneLib.SetDeviceUsesRenderPriority
    SetDeviceUsesRenderPriority.argtypes    = [c_int, c_int]

    GetRenderPriority                       = m_OctaneLib.GetRenderPriority
    GetRenderPriority.restype               = c_int
    
    SetRenderPriority                       = m_OctaneLib.SetRenderPriority
    SetRenderPriority.argtypes              = [c_int]
            
    SaveDeviceRenderConfig                  = m_OctaneLib.SaveDeviceRenderConfig
       
    # Pick Functions    
    PickFocus                               = m_OctaneLib.PickFocus
    PickFocus.argtypes                      = [c_int, c_int]
    PickFocus.restype                       = c_float

    PickMaterial                            = m_OctaneLib.PickMaterial
    PickMaterial.argtypes                   = [c_int, c_int]
    PickMaterial.restype                    = OCTANE_NODE_PTR

    PickWhiteBalance                        = m_OctaneLib.PickWhiteBalance
    PickWhiteBalance.argtypes               = [c_int, c_int]
    PickWhiteBalance.restype                = Float3
    
    # Rendering Statistics    
    GetResultStatistics                     = m_OctaneLib.GetResultStatistics
    GetResultStatistics.argtypes            = [POINTER(ResultStatistics)]
    
    GetTexturesStatistic                    = m_OctaneLib.GetTexturesStatistic
    GetTexturesStatistic.argtypes           = [POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), 
                                                POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), 
                                                POINTER(c_uint), POINTER(c_uint)]
    GetTexturesStatistic.restype            = c_bool

    GetMemoryUsage                          = m_OctaneLib.GetMemoryUsage
    GetMemoryUsage.argtypes                 = [POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), 
                                                POINTER(c_uint), POINTER(c_uint), POINTER(c_uint), 
                                                POINTER(c_uint)]
    GetMemoryUsage.restype                  = c_bool
            
    # Nodes
    GetRenderTargetNode                     = m_OctaneLib.GetRenderTargetNode
    GetRenderTargetNode.restype             = OCTANE_NODE_PTR

    SetGeometryGroupPinCount                = m_OctaneLib.SetGeometryGroupPinCount
    SetGeometryGroupPinCount.argtypes       = [OCTANE_NODE_PTR, c_int]

    SetGeometryGroupPin                     = m_OctaneLib.SetGeometryGroupPin
    SetGeometryGroupPin.argtypes            = [OCTANE_NODE_PTR, c_int, OCTANE_NODE_PTR]
    
    # Same as Octane Standalone LUA pinNode = node.getNodeInfo() - returned just the node type
    GetNodeType                             = m_OctaneLib.GetNodeType
    GetNodeType.argtypes                    = [OCTANE_NODE_PTR]
    GetNodeType.restype                     = c_int
    
    # Same as Octane Standalone LUA pinNode = node.getPinCount()
    GetPinCount                             = m_OctaneLib.GetPinCount
    GetPinCount.argtypes                    = [OCTANE_NODE_PTR]
    GetPinCount.restype                     = c_int
    
    # Same as Octane Standalone LUA pinNode = node.getPinInfo() - returned just the pin name
    GetPinName                              = m_OctaneLib.GetPinName
    GetPinName.argtypes                     = [OCTANE_NODE_PTR, c_int]
    GetPinName.restype                      = STRING
    
    GetPinInfo                              = m_OctaneLib.GetPinInfo
    GetPinInfo.argtypes                     = [OCTANE_NODE_PTR, STRING, POINTER(PinInfo)]
    
    # Same as Octane Standalone LUA pinNode = node.getConnectedNode(node, pinName)        
    GetPinNode                              = m_OctaneLib.GetPinNode
    GetPinNode.argtypes                     = [OCTANE_NODE_PTR, c_int]
    GetPinNode.restype                      = OCTANE_NODE_PTR
    
    # Same as Octane Standalone LUA pinNode = node.getConnectedNode(node, pinName)        
    GetPinNodeByName                        = m_OctaneLib.GetPinNodeByName
    GetPinNodeByName.argtypes               = [OCTANE_NODE_PTR, STRING]
    GetPinNodeByName.restype                = OCTANE_NODE_PTR
    
    GetStringAttribute                      = m_OctaneLib.GetStringAttribute
    GetStringAttribute.argtypes             = [OCTANE_NODE_PTR, STRING]
    GetStringAttribute.restype              = STRING
    
    SetStringAttribute                      = m_OctaneLib.SetStringAttribute
    SetStringAttribute.argtypes             = [OCTANE_NODE_PTR, STRING, STRING]

    # Same as Octane Standalone LUA node.setAttribute(node, A_VALUE, floatValue, true)        
    SetFloatAttribute                       = m_OctaneLib.SetFloatAttribute
    SetFloatAttribute.argtypes              = [OCTANE_NODE_PTR, c_float, STRING]
    
    # Same as Octane Standalone LUA float3Value = node.getAttribute(node, A_VALUE)        
    GetFloat3Param                          = m_OctaneLib.GetFloat4Param
    GetFloat3Param.argtypes                 = [OCTANE_NODE_PTR, c_int]
    GetFloat3Param.restype                  = Float3
    
    # Same as Octane Standalone LUA node.setAttribute(node, A_VALUE, float3Value, true)        
    SetFloat3Param                          = m_OctaneLib.SetFloat3Param
    SetFloat3Param.argtypes                 = [OCTANE_NODE_PTR, c_float, c_float, c_float] 

    # Same as Octane Standalone LUA intValue = node.getAttribute(node, A_VALUE)        
    GetInt3Param                            = m_OctaneLib.GetInt3Param
    GetInt3Param.argtypes                   = [OCTANE_NODE_PTR, c_int]
    GetInt3Param.restype                    = Int3
    
    # Same as Octane Standalone LUA node.setAttribute(node, A_VALUE, intValue, true)        
    SetInt3Param                            = m_OctaneLib.SetInt3Param
    SetInt3Param.argtypes                   = [OCTANE_NODE_PTR, c_int, c_int, c_int]

    # Used to set a Scatter node transform array
    SetFloatArrayParam                      = m_OctaneLib.SetFloatArrayParam
    SetFloatArrayParam.argtypes             = [OCTANE_NODE_PTR, c_int, POINTER(c_float), c_int]
        
    # Same as Octane Standalone LUA node.destroy()
    DeleteNode                              = m_OctaneLib.DeleteNode
    DeleteNode.argtypes                     = [OCTANE_NODE_PTR]
    
    # Same as Octane Standalone LUA node.create(nodeType)
    CreateNode                              = m_OctaneLib.CreateNode
    CreateNode.argtypes                     = [c_int]
    CreateNode.restype                      = OCTANE_NODE_PTR
    
    # Same as Octane Standalone LUA node.connectToIx()
    SetPinNodeByPinNumber                   = m_OctaneLib.SetPinNode
    SetPinNodeByPinNumber.argtypes          = [OCTANE_NODE_PTR, c_int, OCTANE_NODE_PTR]
    
    # Same as Octane Standalone LUA node.connectTo()
    SetPinNodeByPinName                     = m_OctaneLib.SetPinNodeByName
    SetPinNodeByPinName.argtypes            = [OCTANE_NODE_PTR, STRING, OCTANE_NODE_PTR]
        
    # Graph / log
    OpenLogWindow                           = m_OctaneLib.OpenLogWindow
    ShowGraphWindow                         = m_OctaneLib.ShowGraphWindow

    CreateInternalNode                      = m_OctaneLib.CreateInternalNode
    CreateInternalNode.argtypes             = [OCTANE_NODE_PTR, STRING, c_int]
    CreateInternalNode.restype              = OCTANE_NODE_PTR

    CreateInternalNodeByPinNum              = m_OctaneLib.CreateInternalNodeByPinNum
    CreateInternalNodeByPinNum.argtypes     = [OCTANE_NODE_PTR, c_int, c_int]
    CreateInternalNodeByPinNum.restype      = OCTANE_NODE_PTR
    
    NodeHasPin                              = m_OctaneLib.NodeHasPin
    NodeHasPin.argtypes                     = [OCTANE_NODE_PTR, STRING]
    NodeHasPin.restype                      = c_bool

    GetOctaneVersion                        = m_OctaneLib.GetOctaneVersion
    GetOctaneVersion.restype                = c_int

    # Texture maps
    ReloadTexturemaps                       = m_OctaneLib.ReloadTexturemaps
    ReloadTexturemaps.argtypes              = [OCTANE_NODE_PTR]
   
    # Same as Octane Standalone LUA nodeGraph.updateTime()    
    UpdateTime                              = m_OctaneLib.UpdateTime
    UpdateTime.argtypes                     = [c_float]
    
    # Same as Octane Standalone LUA render.setShutterTime()    
    SetShutterSpeed                         = m_OctaneLib.SetShutterSpeed
    SetShutterSpeed.argtypes                = [c_float]

    # Same as Octane Standalone LUA node.PROPS_NODE_ITEM graphOwned()    
    IsExternal                              = m_OctaneLib.IsExternal 
    IsExternal.argtypes                     = [OCTANE_NODE_PTR] 
    IsExternal.restype                      = c_bool
 
    # Same as Octane Standalone LUA node.PROPS_NODE_ITEM name()    
    SetNodeName                             = m_OctaneLib.SetNodeName 
    SetNodeName.argtypes                    = [OCTANE_NODE_PTR, STRING] 

    # Same as Octane Standalone LUA node.PROPS_NODE_ITEM name()    
    GetNodeName                             = m_OctaneLib.GetNodeName 
    GetNodeName.argtypes                    = [OCTANE_NODE_PTR] 
    GetNodeName.restype                     = STRING

    # Same as Octane Standalone LUA render.setSubSampleMode()
    SetSubsampling                          = m_OctaneLib.SetSubsampling
    SetSubsampling.argtypes                 = [c_int]   

    # Same as Octane Standalone LUA render.setClayMode()
    SetClayMode                             = m_OctaneLib.SetClayMode
    SetClayMode.argtypes                    = [c_int]   
    
    SetRenderRegion                         = m_OctaneLib.SetRenderRegion
    SetRenderRegion.argtypes                = [c_bool, c_int, c_int, c_int, c_int]   

    LoadMaterialFromOCS                     = m_OctaneLib.LoadMaterialFromOCS
    LoadMaterialFromOCS.argtypes            = [STRING]
    LoadMaterialFromOCS.restype             = OCTANE_NODE_PTR
    
    SaveMaterialToLocalDb                   = m_OctaneLib.SaveMaterialToLocalDb
    SaveMaterialToLocalDb.argtypes          = [OCTANE_NODE_PTR, STRING, STRING]
  
    SetAsyncTonemapRenderPass               = m_OctaneLib.SetAsyncTonemapRenderPass
    SetAsyncTonemapRenderPass.argtypes      = [c_int]
    
    SaveMaterialToString                    = m_OctaneLib.SaveMaterialToString
    SaveMaterialToString.argtype            = [OCTANE_NODE_PTR]
    SaveMaterialToString.restype            = STRING
           
    LoadMaterialFromString                  = m_OctaneLib.LoadMaterialFromString
    LoadMaterialFromString.argtype          = [STRING, STRING]
    LoadMaterialFromString.restype          = OCTANE_NODE_PTR
    
    IsARenderPassPin                        = m_OctaneLib.IsARenderPassPin
    IsARenderPassPin.argtype                = [STRING]
    IsARenderPassPin.restype                = c_bool
       
    if not IsStarted():
        print "WARNING: You need to start an Octane render prior to issuing any octane module calls."
